#include "Ball.h"
#include "NxMeshCooker.h"

using namespace Ogre;

Ball::Ball(Ogre::SceneManager* sceneManager, NxScene* mPhysicsScene, NxPhysicsSDK* mPhysicsSDK, int id){
	ogreNode = sceneManager->getRootSceneNode()->createChildSceneNode();
	Entity *ballEnt = sceneManager->createEntity("ball"+StringConverter::toString(id),"strippedBall.mesh");
	ogreNode->attachObject(ballEnt);
	ogreNode->setScale(1,1,1);
	ogreNode->setPosition(30, 0, 60);

	material.restitution = 1;
	material.dynamicFriction = 0.1;
	material.staticFriction = 0.1;
	material.restitutionCombineMode = NX_CM_MIN;
	material.frictionCombineMode = NX_CM_MAX;
	mat = mPhysicsScene->createMaterial(material);
	sphereDesc.materialIndex = mat->getMaterialIndex();
	mat = mPhysicsScene->getMaterialFromIndex(0);
	mat->setFrictionCombineMode(NX_CM_MAX);
	mat->setRestitutionCombineMode(NX_CM_MIN);

	sphereDesc.radius = 1;
	actorDesc.shapes.pushBack(&sphereDesc);
	actorDesc.body = &bodyDesc;
	actorDesc.density = 100;
	actorDesc.globalPose.t = NxVec3(30, 0, 60);
	actor = mPhysicsScene->createActor(actorDesc);
}

void Ball::update(){
	ogreNode->setOrientation(actor->getGlobalOrientationQuat().w,
							actor->getGlobalOrientationQuat().x,
							actor->getGlobalOrientationQuat().y,
							actor->getGlobalOrientationQuat().z);
	ogreNode->setPosition(actor->getGlobalPosition().x,
							actor->getGlobalPosition().y,
							actor->getGlobalPosition().z);

	//NxVec3 velocity = actor->getLinearVelocity();
	//LogManager::getSingleton().logMessage("x:"+StringConverter::toString(velocity.x)+" y:"+StringConverter::toString(velocity.y)+" z:"+StringConverter::toString(velocity.z));
}

void Ball::reset(){
	actor->raiseBodyFlag(NX_BF_DISABLE_GRAVITY);
	actor->setLinearVelocity(NxVec3(0,0,0));
	actor->setGlobalPosition(NxVec3(30, 0, 60));
}

void Ball::start(){
	actor->clearBodyFlag(NX_BF_DISABLE_GRAVITY);
	//actor->setLinearVelocity(NxVec3(0,0,0));
	actor->setGlobalPosition(NxVec3(30, 0, 60));
}
void Ball::travel(int time){
	if(time==1955){
		actor->setLinearVelocity(NxVec3(0,0,0));
		actor->setGlobalPosition(NxVec3(15, 0, -8.9));
	}
	else if(time==1985){
		actor->setLinearVelocity(NxVec3(0,0,0));
		actor->setGlobalPosition(NxVec3(-20.4, 0, 4.5));
	}
}