#pragma once
#include "stdafx.h"
#include "Ogre.h"
#define OIS_DYNAMIC_LIB
#include <OIS/OIS.h>
#include "NxPhysics.h"
#include "PhysicsHandler.h"

using namespace Ogre;

// class defs
class HUD;
class AnimController;
class ControlDeviceHandler;

// globals
GameController* game;
Root* ogreRoot;
Ogre::Camera* camera;
SceneManager* sceneManager;
RenderWindow* renderWindow;
OIS::InputManager* OISInputManager;
PhysicsHandler* physicsHandler;
Triggerable* triggerable;
HUD* hud;

class HUD : public FrameListener{
public:
	float delayTime;
	HUD(){
		delayTime = 0;
		Ogre::Overlay* mainOverlay = Ogre::OverlayManager::getSingleton().getByName("MainOverlay");
		mainOverlay->show();
	}
	bool frameStarted(const FrameEvent& evt){
		if (delayTime >= 0)
			delayTime -= evt.timeSinceLastFrame;

		Ogre::OverlayElement* pointText = Ogre::OverlayManager::getSingleton().getOverlayElement("Points");
		pointText->setCaption(Ogre::StringConverter::toString(game->getPoints()));
		Ogre::OverlayElement* multText = Ogre::OverlayManager::getSingleton().getOverlayElement("Mult");
		multText->setCaption("x"+Ogre::StringConverter::toString(game->getMult()));
		Ogre::OverlayElement* ballsText = Ogre::OverlayManager::getSingleton().getOverlayElement("Balls");
		ballsText->setCaption(Ogre::StringConverter::toString(game->getAvBalls())+" balls");
		
		Ogre::OverlayElement* gameText = Ogre::OverlayManager::getSingleton().getOverlayElement("Gamestate");
		gameText->setCaption(game->getGamestate());
		
		if(delayTime >= 0){
			gameText->setCaption(game->getGamestate());
		}
		else{
			gameText->setCaption("");
		}
		
		return true;
	}
	void setDelay(float f){
		delayTime = f;
	}
};

// Lever Animation Controller
class AnimController : public FrameListener{
protected:
	float speed;
	float maxUp;
	float maxDown;
	Vector3 axis;
public:
	AnimController(){
		speed = 9.0f;
		maxUp = 0.35;
		maxDown = -0.2;
		axis = Vector3::UNIT_Y;
	}
	bool frameStarted(const FrameEvent& evt){
		SceneNode* tmplvr = physicsHandler->getLever()->getNode();
		float currState = ((SceneNode*) tmplvr->getChild(1))->getOrientation().y;
		float currState2 = ((SceneNode*) tmplvr->getChild(0))->getOrientation().y;

		if(game->dir == 1){
			if(currState >= maxUp){
				game->dir = 2;				
			}
			else{
				((SceneNode*)(tmplvr->getChild(1)))->rotate(axis, Radian(evt.timeSinceLastFrame * speed));
			}
		}
		else if(game->dir == 2){
			if(currState <= maxDown){
				game->dir = 0;				
			}
			else{
				((SceneNode*)(tmplvr->getChild(1)))->rotate(axis, Radian(evt.timeSinceLastFrame * -speed));
			}
		}

		if(game->dir2 == 1){
			if(currState2 <= -maxUp){
				game->dir2 = 2;				
			}
			else{
				((SceneNode*)(tmplvr->getChild(0)))->rotate(axis, Radian(evt.timeSinceLastFrame * -speed));
			}
		}
		else if(game->dir2 == 2){
			if(currState2 >= -maxDown){
				game->dir2 = 0;				
			}
			else{
				((SceneNode*)(tmplvr->getChild(0)))->rotate(axis, Radian(evt.timeSinceLastFrame * speed));
			}
		}
		return true;
	}
};


// Input control handler
class ControlDeviceHandler : public FrameListener{
protected:
	OIS::Keyboard* mKeyboard;
	OIS::Mouse* mMouse;
	float mTimeUntilNextToggle;
	
public:	
	ControlDeviceHandler(){		
		LogManager::getSingletonPtr()->logMessage("*** Initializing OIS ***");
		OIS::ParamList pl;
		size_t windowHnd = 0;
		renderWindow->getCustomAttribute("WINDOW", &windowHnd);
		std::ostringstream windowHndStr;
		windowHndStr << windowHnd;
		pl.insert(std::make_pair(std::string("WINDOW"), windowHndStr.str()));
		OISInputManager = OIS::InputManager::createInputSystem( pl );
		mKeyboard = static_cast<OIS::Keyboard*>(OISInputManager->createInputObject( OIS::OISKeyboard, false ));
		mMouse = static_cast<OIS::Mouse*>(OISInputManager->createInputObject( OIS::OISMouse, false ));
	}
	~ControlDeviceHandler()	{
		LogManager::getSingletonPtr()->logMessage("*** Shutting down OIS ***");
		OIS::InputManager::destroyInputSystem(OISInputManager);
	}	
	bool frameStarted(const FrameEvent& evt){
		if (mTimeUntilNextToggle >= 0)
		mTimeUntilNextToggle -= evt.timeSinceLastFrame;
		
		mKeyboard->capture();
		mMouse->capture();

		if( mKeyboard->isKeyDown(OIS::KC_ESCAPE))
			return false;
		if( mKeyboard->isKeyDown(OIS::KC_R) && mTimeUntilNextToggle < 0){
			game->resetGame();
			mTimeUntilNextToggle = 0.25f;
		}
		if( mKeyboard->isKeyDown(OIS::KC_SPACE) && mTimeUntilNextToggle < 0){
			game->startBall();
			mTimeUntilNextToggle = 0.25f;
		}
		if( mKeyboard->isKeyDown(OIS::KC_BACK) && mTimeUntilNextToggle < 0){
			game->resetBall();
			mTimeUntilNextToggle = 0.25f;
		}
		if( mKeyboard->isKeyDown(OIS::KC_RSHIFT)){
			game->dir2 = 1;
			//mTimeUntilNextToggle = 0.001f;
		}
		if( mKeyboard->isKeyDown(OIS::KC_LSHIFT)){
			game->dir = 1;
			//mTimeUntilNextToggle = 0.001f;
		}

		/*
		// camera movement
		float mousesensitivity = 1.0f;
		float cameraMoveSpeed = 50.0f * evt.timeSinceLastFrame;
		float cameraRotateSpeed = 50.0f * evt.timeSinceLastFrame;
		Vector3 campos = camera->getPosition();
		Vector3 camdir = camera->getDirection();
		Vector3 camright = camera->getRight();
		Vector3 camup = camera->getUp();
		Vector3 cammove(0,0,0);
		const OIS::MouseState &ms = mMouse->getMouseState();
		if(ms.buttonDown(OIS::MB_Right)){
			cammove += camright * ms.X.rel * 0.01f;
			cammove += camup * ms.Y.rel * 0.01f;
		}
		else{
			camera->yaw(Radian(-cameraRotateSpeed * mousesensitivity * ms.X.rel * 0.01f));
			camera->pitch(Radian(-cameraRotateSpeed * mousesensitivity * ms.Y.rel * 0.01f));
		}
		if( mKeyboard->isKeyDown(OIS::KC_UP) || mKeyboard->isKeyDown(OIS::KC_W))
			cammove += camdir;
		if( mKeyboard->isKeyDown(OIS::KC_DOWN) || mKeyboard->isKeyDown(OIS::KC_S))
			cammove -= camdir;
		if( mKeyboard->isKeyDown(OIS::KC_A))
			cammove -= camright;
		if( mKeyboard->isKeyDown(OIS::KC_D))
			cammove += camright;
		if( mKeyboard->isKeyDown(OIS::KC_Q))
			cammove += camup;
		if( mKeyboard->isKeyDown(OIS::KC_E))
			cammove -= camup;
		cammove.normalise();
		camera->setPosition(campos + cammove * cameraMoveSpeed);
		if( mKeyboard->isKeyDown(OIS::KC_LEFT))
			camera->rotate(Vector3::UNIT_Y,Radian(cameraRotateSpeed * 0.02f));
		if( mKeyboard->isKeyDown(OIS::KC_RIGHT))
			camera->rotate(Vector3::UNIT_Y, Radian(-cameraRotateSpeed * 0.02f));
		camright = camera->getRight();
		if( mKeyboard->isKeyDown(OIS::KC_PGUP))
			camera->rotate(camright, Radian(cameraRotateSpeed * 0.02f));
		if( mKeyboard->isKeyDown(OIS::KC_PGDOWN))
			camera->rotate(camright, Radian(-cameraRotateSpeed * 0.02f));
		//LogManager::getSingleton().logMessage("x:"+StringConverter::toString(camera->getPosition().x)+" y:"+StringConverter::toString(camera->getPosition().y)+" z:"+StringConverter::toString(camera->getPosition().z));
		*/

		return true;
	}
};

GameController::GameController(){
	gamestate = "Play!";
	dir = 0;
	dir2 = 0;
	points = 0;
	mult = 1;
	avballs = 5;
}
void GameController::startBall(){
	if(gamestate == "Game Over!"){
		resetGame();
		physicsHandler->startBall();
	}
	else{
		physicsHandler->startBall();
	}

	setGamestate("", 0);
}
void GameController::resetBall(){
	decBalls();
	physicsHandler->resetBall();
}
void GameController::addPoints(int _points){
	points = points+(mult*_points);
}
int GameController::getPoints(){
	return points;
}
void GameController::resetPoints(){
	points = 0;
}
void GameController::setMult(int _mult){
	mult = _mult;
}
int GameController::getMult(){
	return mult;
}
void GameController::decBalls(){
	if(avballs == 0){
		setGamestate("Game Over!", 2.0);
	}
	mult = 1;
	avballs = avballs-1;
}
void GameController::resetGame(){
	setGamestate("Play!", 2.0);
	physicsHandler->resetBall();
	points = 0;
	mult = 1;
	avballs = 5;
}
int GameController::getAvBalls(){
	return avballs;
}
void GameController::setGamestate(std::string state, float delay){
	hud->setDelay(delay);
	gamestate = state;
}
std::string GameController::getGamestate(){
	return gamestate;
}
void GameController::timeTravel(int time){
	setGamestate("Time traveler!", 2.0);
	setMult(getMult()*2);
	physicsHandler->travelBall(time);
}

void setupPostProc()
{
/*
	renderWindow->getViewport(0)->setClearEveryFrame(true, FBT_DEPTH);
	Root::getSingleton().getRenderSystem()->clearFrameBuffer( FBT_COLOUR, ColourValue::Black);


	CompositorManager::getSingleton().
			addCompositor(renderWindow->getViewport(0), "Glow");
	CompositorManager::getSingleton().
			setCompositorEnabled(renderWindow->getViewport(0), "Glow", true);

	CompositorManager::getSingleton().addCompositor(renderWindow->getViewport(0),
														"MotionBlur");
	CompositorManager::getSingleton().setCompositorEnabled(renderWindow->getViewport(0),
													"MotionBlur", true);	
*/
}


// init scene, camera, lights, game
void init(){
	// create game
	game = new GameController();

	camera = sceneManager->createCamera("Main Camera");       
	camera->setPosition(0, 70, 90);	// XYZ
	camera->lookAt(0, -10, 10);
	camera->setFarClipDistance(500);
	camera->setNearClipDistance(0.1);

	Viewport* viewport = renderWindow->addViewport(camera);
	renderWindow->setActive(true); 

	sceneManager->setAmbientLight(ColourValue(0, 0, 0, 1.0));

	Light* light = sceneManager->createLight("pointlight1");
	light->setType(Light::LT_POINT);
	light->setAttenuation(3000,0,0,1);
	light->setPowerScale(6000);
	light->setCastShadows(false);
	light->setPosition(0, 50, -10);	// XYZ

	Light* light2 = sceneManager->createLight("pointlight2");
	light2->setType(Light::LT_POINT);
	light2->setAttenuation(3000,0,0,1);
	light2->setPowerScale(6000);
	light2->setCastShadows(false);
	light2->setPosition(0, 50, 40);	// XYZ

	sceneManager->setShadowTechnique(SHADOWTYPE_STENCIL_MODULATIVE);
	sceneManager->setShadowColour(ColourValue(0.5,0.5,0.5));

	// add physX
	physicsHandler = new PhysicsHandler(sceneManager, game);
	ogreRoot->addFrameListener(physicsHandler);
	// add hud
	ogreRoot->addFrameListener(hud = new HUD());
	// add animations
	ogreRoot->addFrameListener(new AnimController());
	// add controls
	ogreRoot->addFrameListener(new ControlDeviceHandler());
	// add post proc
	setupPostProc();
}

// main function
int _tmain(int argc, _TCHAR* argv[]){
	ogreRoot = new Root("","");
	#ifdef _DEBUG
		ogreRoot->loadPlugin("RenderSystem_Direct3D9_d");
		ogreRoot->loadPlugin("Plugin_OctreeSceneManager_d");
	#else
		ogreRoot->loadPlugin("RenderSystem_Direct3D9");
		ogreRoot->loadPlugin("Plugin_OctreeSceneManager");
	#endif

	// setup render system
	RenderSystemList* list = ogreRoot->getAvailableRenderers(); 
	ogreRoot->setRenderSystem(list->at(0));
	ogreRoot->initialise(false);
	renderWindow = ogreRoot->createRenderWindow("FlippX", 800, 600, false);

	// create sceneManager
	sceneManager = ogreRoot->createSceneManager("OctreeSceneManager"); 

	// media file loaction
	ResourceGroupManager::getSingleton().addResourceLocation("media","FileSystem","General", false);
	ResourceGroupManager::getSingleton().initialiseAllResourceGroups(); 

	// setup scene
	init();

	//ogreRoot->startRendering();
	while(true)
	{
		if(renderWindow->isClosed())
		{
			ogreRoot->shutdown();
			break;
		}
		WindowEventUtilities::messagePump();
		
		if(!ogreRoot->renderOneFrame())
		{
			ogreRoot->shutdown();
			break;
		}
	}

	return 0;
}

